@@ -85,13 +85,16 @@ get.thinned.tfr.mcmc <- function(mcmc.set, thin=1, burnin=0) {
 	if(file.exists(dir.name)) return(get.tfr.mcmc(dir.name))
 	return(NULL)
 }
-	
-create.thinned.tfr.mcmc <- function(mcmc.set, thin=1, burnin=0, output.dir=NULL, verbose=TRUE, uncertainty=FALSE) {
+
+create.thinned.tfr.mcmc <- function(mcmc.set, thin=1, burnin=0, output.dir=NULL, verbose=TRUE, uncertainty=FALSE,
+                                    update.with.countries = NULL) {
 	#Return a thinned mcmc.set object with burnin removed and all chanins collapsed into one
-  mcthin <- max(sapply(mcmc.set$mcmc.list, function(x) x$thin))
+    mcthin <- max(sapply(mcmc.set$mcmc.list, function(x) x$thin))
 	thin <- max(c(thin, mcthin))
 	meta <- mcmc.set$meta
 	total.iter <- get.stored.mcmc.length(mcmc.set$mcmc.list, burnin=burnin)
+	total.iter <- min(total.iter, get.stored.mcmc.length.extra(mcmc.set$meta, update.with.countries, 
+	                                                            nr.chains = length(mcmc.set$mcmc.list), burnin = burnin), na.rm = TRUE)
 	meta$is.thinned <- TRUE
 	meta$parent.iter <- get.total.iterations(mcmc.set$mcmc.list, burnin)
 	meta$parent.meta <- mcmc.set$meta
@@ -104,6 +107,9 @@ create.thinned.tfr.mcmc <- function(mcmc.set, thin=1, burnin=0, output.dir=NULL,
 			paste('thinned_mcmc', thin, burnin, sep='_'))
 	if(!file.exists(meta$output.dir)) 
 		dir.create(meta$output.dir, recursive=TRUE)
+	
+	if(!is.null(update.with.countries)) meta$one.use.only <- TRUE # thinned traces might get out of sync with other countries; thus recompute next time
+	
 	store.bayesTFR.meta.object(meta, meta$output.dir)
 	
 	thin.index <- if(thin > mcthin) unique(round(seq(1, total.iter, by=thin/mcthin))) else 1:total.iter
@@ -139,11 +145,12 @@ create.thinned.tfr.mcmc <- function(mcmc.set, thin=1, burnin=0, output.dir=NULL,
 	if(verbose) cat('done.\nStoring country-specific parameters ...')
 	par.names.cs <- tfr.parameter.names.cs(trans=FALSE)
 	if (uncertainty) par.names.cs <- c(par.names.cs, 'tfr')
-	for (country in mcmc.set$meta$id_DL){
+	DLcntries <- if(!is.null(update.with.countries)) update.with.countries[update.with.countries %in% mcmc.set$meta$id_DL] else mcmc.set$meta$id_DL
+	for (country in DLcntries){
 		country.obj <- get.country.object(country, mcmc.set$meta, index=TRUE)
 		if (country.obj$index %in% mcmc.set$meta$extra)
 		{
-		  nr.points.cs <- floor(mcmc.set$meta$extra_iter[country.obj$index] / mcmc.set$meta$extra_thin[country.obj$index])
+		  nr.points.cs <- floor((mcmc.set$meta$extra_iter[country.obj$index]-burnin) / mcmc.set$meta$extra_thin[country.obj$index])
 		  nr.points.cs <- nr.points.cs * length(mcmc.set$mcmc.list)
 		  if (nr.points.cs >= nr.points) thin.index.cs <- round(seq(1, nr.points.cs, length.out = nr.points))
 		  else 
@@ -168,8 +175,10 @@ create.thinned.tfr.mcmc <- function(mcmc.set, thin=1, burnin=0, output.dir=NULL,
 		}
 	}
 	if (mcmc.set$meta$nr_countries > mcmc.set$meta$nr_countries_estimation) {
-		.update.thinned.extras(mcmc.set, (mcmc.set$meta$nr_countries_estimation+1):mcmc.set$meta$nr_countries,
-								burnin=burnin, nr.points=nr.points, dir=outdir.thin.mcmc, verbose=verbose)
+	    cindex <- if(!is.null(update.with.countries)) update.with.countries[update.with.countries > mcmc.set$meta$nr_countries_estimation] else 
+	        (mcmc.set$meta$nr_countries_estimation+1):mcmc.set$meta$nr_countries
+	    if(length(cindex) > 0)
+		    .update.thinned.extras(mcmc.set, cindex, burnin=burnin, nr.points=nr.points, dir=outdir.thin.mcmc, verbose=verbose)
 	}
 	invisible(structure(list(meta=meta, mcmc.list=list(thinned.mcmc)), class='bayesTFR.mcmc.set'))
 }
@@ -252,48 +261,53 @@ get.tfr.prediction <- function(mcmc=NULL, sim.dir=NULL, mcmc.dir=NULL) {
 	return(pred)
 }
 
-.get.model.est <- function(excl.what = c(), mcmc.list=NULL, country.code=NULL, ISO.code=NULL, sim.dir=NULL,
-                           remove.duplicates = TRUE) {
-    e <- new.env()
-    data('iso3166', envir=e)
-    iso3166 <- e$iso3166
+.get.model.est <- function(excl.what = c(), mcmc.list=NULL, country=NULL, sim.dir=NULL,
+                           remove.duplicates = TRUE, country.code = deprecated(), ISO.code = deprecated()) {
+
+    if (lifecycle::is_present(country.code)) {
+        lifecycle::deprecate_warn("7.1-0", "tfr.bias.sd(country.code)", "tfr.bias.sd(country)",
+                                  details = "The same applies to get.bias.model() and get.sd.model()")
+        country <- country.code
+    }
+    if (lifecycle::is_present(ISO.code)) {
+        lifecycle::deprecate_warn("7.1-0", "tfr.bias.sd(ISO.code)", "tfr.bias.sd(country)",
+                                  details = "The same applies to get.bias.model() and get.sd.model()")
+        country <- ISO.code
+    }
     if (is.null(mcmc.list)) 
         mcmc.list <- get.tfr.mcmc(sim.dir)
     if (is.null(mcmc.list)) {
         warning('MCMC does not exist.')
         return(NULL)
     }
-    if (!mcmc.list$mcmc.list[[1]]$uncertainty) 
+    if (is.null(mcmc.list$mcmc.list[[1]]$uncertainty) || !mcmc.list$mcmc.list[[1]]$uncertainty) 
     {
         stop("MCMC does not consider uncertainty of past TFR.")
     }
-    if (is.null(country.code))
-        country.code <- iso3166$uncode[iso3166$charcode3 == ISO.code]
-    
-    country.obj <- get.country.object(country.code, mcmc.list$meta)
-    if (country.obj$index %in% mcmc.list$meta$extra) model_est <- mcmc.list$meta$raw_data_extra[[country.obj$index]]
-    else model_est <- mcmc.list$meta$raw_data.original[mcmc.list$meta$raw_data.original$country_code == country.obj$code,]
+    if(is.null(country)) stop("Country must be given.")
+    meta <- mcmc.list$meta
+    country.obj <- get.country.object(country, meta)
+    if (country.obj$index %in% meta$extra) model_est <- meta$raw_data_extra[[country.obj$index]]
+    else model_est <- meta$raw_data.original[meta$raw_data.original$country_code == country.obj$code,]
     model_est <- model_est[, !(names(model_est) %in% c('country_code', 'Country.or.area', 'year', 'tfr', 'country_index', 'eps', excl.what))]
     if(remove.duplicates)
         model_est <- model_est[!duplicated(model_est),]
-    return(list(table = model_est, cntry.index = country.obj$index, meta = mcmc.list$meta))
+    return(list(table = model_est, cntry.index = country.obj$index, meta = meta))
 }
 
-get.bias.model <- function(mcmc.list=NULL, country.code=NULL, ISO.code=NULL, sim.dir=NULL) {
-    model_est <- .get.model.est(excl.what = "std", mcmc.list = mcmc.list, country.code = country.code, 
-                                ISO.code = ISO.code, sim.dir = sim.dir)
+get.bias.model <- function(mcmc.list=NULL, country=NULL, sim.dir=NULL, ...) {
+    model_est <- .get.model.est(excl.what = "std", mcmc.list = mcmc.list, country = country, sim.dir = sim.dir, ...)
     return(list(model = model_est$meta$bias_model[[model_est$cntry.index]], table=model_est$table))
 }
 
-get.std.model <- function(mcmc.list=NULL, country.code=NULL, ISO.code=NULL, sim.dir=NULL) {
-    model_est <- .get.model.est(excl.what = "bias", mcmc.list = mcmc.list, country.code = country.code, 
-                                ISO.code = ISO.code, sim.dir = sim.dir)
+get.std.model <- function(mcmc.list=NULL, country=NULL, sim.dir=NULL, ...) {
+    model_est <- .get.model.est(excl.what = "bias", mcmc.list = mcmc.list, country = country, sim.dir = sim.dir, ...)
     return(list(model=model_est$meta$std_model[[model_est$cntry.index]], table=model_est$table))
 }
 
-tfr.bias.sd <- function(mcmc.list=NULL, country.code=NULL, ISO.code=NULL, sim.dir=NULL) {
-    model_est <- .get.model.est(mcmc.list = mcmc.list, country.code = country.code, 
-                                ISO.code = ISO.code, sim.dir = sim.dir, remove.duplicates = FALSE)
+tfr.bias.sd <- function(mcmc.list=NULL, country=NULL, sim.dir=NULL, ...) {
+    model_est <- .get.model.est(mcmc.list = mcmc.list, country = country, 
+                                sim.dir = sim.dir, remove.duplicates = FALSE, ...)
     covariates <- if (model_est$cntry.index %in% model_est$meta$extra) model_est$meta$extra_covariates[[model_est$cntry.index]] else model_est$meta$covariates
 
     model_est_dt <- data.table::as.data.table(model_est$table)
@@ -305,25 +319,37 @@ tfr.bias.sd <- function(mcmc.list=NULL, country.code=NULL, ISO.code=NULL, sim.di
                 table = model_est_df))
 }
 
-get.tfr.estimation <- function(mcmc.list=NULL, country.code=NULL, ISO.code=NULL, sim.dir=NULL, 
-                               burnin=0, thin = 1, probs=NULL, adjust=TRUE) {
-  e <- new.env()
-  data('iso3166', envir=e)
-  iso3166 <- e$iso3166
+get.tfr.estimation <- function(mcmc.list=NULL, country=NULL, sim.dir=NULL, 
+                               burnin=0, thin = 1, probs=NULL, adjust=TRUE, 
+                               country.code = deprecated(), ISO.code = deprecated()) {
+
+    if (lifecycle::is_present(country.code)) {
+        lifecycle::deprecate_warn("7.1-0", "get.tfr.estimation(country.code)", "get.tfr.estimation(country)")
+        country <- country.code
+    }
+    if (lifecycle::is_present(ISO.code)) {
+        lifecycle::deprecate_warn("7.1-0", "get.tfr.estimation(ISO.code)", "get.tfr.estimation(country)")
+        country <- ISO.code
+    }
+    
+    if(is.character(mcmc.list)) {
+        sim.dir <- mcmc.list
+        mcmc.list <- NULL
+    }
   if (is.null(mcmc.list)) 
     mcmc.list <- get.tfr.mcmc(sim.dir)
   if (is.null(mcmc.list)) {
     warning('MCMC does not exist.')
     return(NULL)
   }
-  if (!mcmc.list$mcmc.list[[1]]$uncertainty) 
-  {
-    stop("MCMC does not consider uncertainty of past TFR.")
-  }
-  if (is.null(country.code))
-    country.code <- iso3166$uncode[iso3166$charcode3 == ISO.code]
-  
-  country.obj <- get.country.object(country.code, mcmc.list$meta)
+    if (is.null(mcmc.list$mcmc.list[[1]]$uncertainty) || !mcmc.list$mcmc.list[[1]]$uncertainty)
+    {
+        stop("MCMC does not consider uncertainty of past TFR.")
+    }
+    if (is.null(country))
+        stop("Country  must be given.")
+
+  country.obj <- get.country.object(country, mcmc.list$meta)
   tfr_table <- get.tfr.parameter.traces.cs(mcmc.list$mcmc.list, country.obj, 'tfr', burnin = burnin, thin=thin)
   if (adjust)
   {
@@ -727,6 +753,18 @@ get.stored.mcmc.length <- function(mcmc.list, burnin=0) {
 	return(sum(sapply(mcmc.list, get.iter)))
 }
 
+get.stored.mcmc.length.extra <- function(meta, countries.index = NULL, nr.chains = 1, burnin=0) {
+    if (is.null(meta$extra) || is.null(countries.index) || !any(countries.index %in% meta$extra)) return(NA)
+    stored.iter <- NA
+    for (country in countries.index){
+        if(! country %in% meta$extra) next
+        stored.iter <- min(stored.iter, 
+                           floor((meta$extra_iter[country] - burnin) / meta$extra_thin[country]) * nr.chains, na.rm = TRUE)
+    }
+    return(stored.iter)
+}
+
+
 do.get.tfr.parameter.traces <- function(is.cs, mcmc.list, par.names, country.obj=NULL, 
 										burnin=0, thinning.index=NULL, thin=NULL) {
 	# get parameter traces either from disk or from memory (if they were already loaded)
@@ -1029,19 +1067,25 @@ get.country.object <- function(country, meta=NULL, country.table=NULL, index=FAL
 	l <- length(codes)
 	found <- TRUE
 	if (is.numeric(country)) {
-		if (index) {
+		if (index) { # index
 			country.idx <- country
 			country.code <- codes[country.idx]
-		} else { 
+		} else { # country code
 			country.code <- country
 			country.idx <- (1:l)[codes==country.code]
 		}
 		country.name <- as.character(names[country.idx])
 		if (length(country.name) == 0) found <- FALSE
-	} else {
-		country.name <- country
-		country.idx <- (1:l)[names==country.name]
-		country.code <- codes[country.idx]
+	} else { # country is character string
+	    if(! country %in% names && nchar(country) <= 3) { # ISO charcode
+	        country.code <- match.iso(country)
+	        country.idx <- (1:l)[codes==country.code]
+	        country.name <- as.character(names[country.idx])
+	    } else { # country name
+		    country.name <- country
+		    country.idx <- (1:l)[names==country.name]
+		    country.code <- codes[country.idx]
+	    }
 		if (length(country.idx) == 0) found <- FALSE
 	}
 	if (!found) 
@@ -1049,6 +1093,15 @@ get.country.object <- function(country, meta=NULL, country.table=NULL, index=FAL
 	return(list(name=country.name, index=country.idx, code=country.code))
 }
 
+match.iso <- local({
+    e <- new.env()
+    data('iso3166', envir=e)
+    get.code <- function(iso) {
+        col <- if(nchar(iso) > 2) "charcode3" else "charcode"
+        return(e$iso3166$uncode[e$iso3166[[col]] == iso])
+    }
+})
+
 country.names <- function(meta, countries=NULL, index=FALSE) {
 	meta <- get.mcmc.meta(meta)
 	if (is.null(countries)) return(meta$regions$country_name)
