@@ -128,7 +128,7 @@ estimate.bias.sd.raw <- function(mcmc)
 }
 
 estimate.bias.sd.original <- function(mcmc, iso.unbiased=NULL, covariates=c('source', 'method'), 
-                                      cont_covariates=NULL)
+                                      cont_covariates=NULL, source.col.name="source", countries = NULL)
 {
   mcmc$meta$raw_data.original$bias <- NA
   mcmc$meta$raw_data.original$std <- NA
@@ -137,7 +137,8 @@ estimate.bias.sd.original <- function(mcmc, iso.unbiased=NULL, covariates=c('sou
     mcmc$meta$bias_model <- list()
     mcmc$meta$std_model <- list()
   }
-  for(country in 1:mcmc$meta$nr_countries)
+  if(is.null(countries)) countries <- 1:mcmc$meta$nr_countries
+  for(country in countries)
   {
     ISO.code <- get.country.object(country, meta=mcmc$meta, index=TRUE)
     if (is.na(ISO.code$code)) next
@@ -168,10 +169,10 @@ estimate.bias.sd.original <- function(mcmc, iso.unbiased=NULL, covariates=c('sou
       }
     }
     
-    m1 <- lm(as.formula(paste0('diff ~ ', regressor)))
+    m1 <- lm(as.formula(paste0('diff ~ ', regressor)), model = FALSE)
     bias <- predict(m1)
     abs.residual <- abs(residuals(m1))
-    m2 <- lm(as.formula(paste0('abs.residual ~ ', regressor)))
+    m2 <- lm(as.formula(paste0('abs.residual ~ ', regressor)), model = FALSE)
     std <- predict(m2) * sqrt(pi/2)
     std[std < 1e-6] <- 0.1
     std[std < (abs(bias) / 2)] <- abs(bias[std < (abs(bias) / 2)]) 
@@ -180,10 +181,10 @@ estimate.bias.sd.original <- function(mcmc, iso.unbiased=NULL, covariates=c('sou
     ## Optional
     if (ISO.code$code %in% iso.unbiased)
     {
-      if ("source" %in% covariates)
+      if (source.col.name %in% covariates)
       {
         index.by.country.vr.estimate <- which((mcmc$meta$raw_data.original$country_code == ISO.code$code) & 
-                                                (mcmc$meta$raw_data.original[, "source"] %in% c("VR", 'Estimate')))
+                                                (mcmc$meta$raw_data.original[, source.col.name] %in% c("VR")))
         mcmc$meta$raw_data.original$bias[index.by.country.vr.estimate] <- 0
         mcmc$meta$raw_data.original$std[index.by.country.vr.estimate] <- 0.016
       }
@@ -192,11 +193,15 @@ estimate.bias.sd.original <- function(mcmc, iso.unbiased=NULL, covariates=c('sou
         warning("Covariate source not found. iso.unbiased is not used.\n")
       }
     }
+    # to save space
+    attr(m1$terms, ".Environment") <- NULL
+    attr(m2$terms, ".Environment") <- NULL
+    #m1$fitted.values <- NULL # cannot be deleted as the summary function would not work
+    #m2$fitted.values <- NULL
+    
     mcmc$meta$bias_model[[country]] <- m1
     mcmc$meta$std_model[[country]] <- m2
   }
-  
-  
   return(mcmc)
 }
 
@@ -215,11 +220,11 @@ get.eps.all.phases <- function(Dlpar, mcmc, country)
     eps_return[1:(mcmc$meta$start_c[country] - 1)] <- mcmc$meta$tfr_all[2:mcmc$meta$start_c[country], country] - 
       mcmc$meta$tfr_all[1:(mcmc$meta$start_c[country]-1), country]
   }
-  if (mcmc$meta$lambda_c[country] < mcmc$meta$T_end)
+  if (mcmc$meta$lambda_c[country] < mcmc$meta$T_end_c[country])
   {
     id3 <- which(mcmc$meta$id_phase3 == country)
-    eps_return[mcmc$meta$lambda_c[country]:(mcmc$meta$T_end - 1)] <- mcmc$meta$tfr_all[(mcmc$meta$lambda_c[country]+1):mcmc$meta$T_end, country] - 
-      mcmc$meta$tfr_all[mcmc$meta$lambda_c[country]:(mcmc$meta$T_end-1), country] * mcmc$rho.c[id3] - (1-mcmc$rho.c[id3]) * mcmc$mu.c[id3]
+    eps_return[mcmc$meta$lambda_c[country]:(mcmc$meta$T_end_c[country] - 1)] <- mcmc$meta$tfr_all[(mcmc$meta$lambda_c[country]+1):mcmc$meta$T_end_c[country], country] - 
+      mcmc$meta$tfr_all[mcmc$meta$lambda_c[country]:(mcmc$meta$T_end_c[country]-1), country] * mcmc$rho.c[id3] - (1-mcmc$rho.c[id3]) * mcmc$mu.c[id3]
   }
   return (eps_return)
 }
@@ -231,9 +236,9 @@ get.sd.all.phases <- function(mcmc, country)
   {
     sd_return[1:(mcmc$meta$start_c[country] - 1)] <- mcmc$sd_eps_tau
   }
-  if (mcmc$meta$lambda_c[country] < mcmc$meta$T_end)
+  if (mcmc$meta$lambda_c[country] < mcmc$meta$T_end_c[country])
   {
-    sd_return[mcmc$meta$lambda_c[country]:(mcmc$meta$T_end - 1)] <- mcmc$sigma.eps
+    sd_return[mcmc$meta$lambda_c[country]:(mcmc$meta$T_end_c[country] - 1)] <- mcmc$sigma.eps
   }
   return (sd_return)
 }
@@ -251,12 +256,12 @@ mcmc.update.tfr <- function(country, mcmc)
   eps_tfr_prop <- eps_tfr_prev
   sd_tfr_prop <- sd_tfr_prev
   if (country %in% mcmc$meta$id_phase3) {id3 <- which(mcmc$meta$id_phase3 == country)}
-  for (year in 1:mcmc$meta$T_end)
+  for (year in 1:mcmc$meta$T_end_c[country])
   {
-    tmp.years <- max(1, year-1):min(year, mcmc$meta$T_end -1)
+    tmp.years <- max(1, year-1):min(year, mcmc$meta$T_end_c[country] -1)
     if (!is.null(mcmc$meta$ar.phase2) && mcmc$meta$ar.phase2)
     {
-      tmp.extra.year <- max(2, year-1):min(year+1, mcmc$meta$T_end -1)
+      tmp.extra.year <- max(2, year-1):min(year+1, mcmc$meta$T_end_c[country] -1)
       tmp.extra.year <- tmp.extra.year[tmp.extra.year > mcmc$meta$start_c[country] & tmp.extra.year < mcmc$meta$lambda_c[country]]
       tmp.more.years <- tmp.years[!(tmp.years %in% tmp.extra.year)]
     }
